#ifndef APP_ERROR_H__
#define APP_ERROR_H__

#include <stdlib.h>
#include "log.h"
#include "ret_code.h"


/** @brief A macro checking the error code and calling the error handler if an error occurred. */
#define APP_ERROR_CHECK(ret_code)                               \
    do {                                                        \
        if (ret_code != RET_SUCCESS)                            \
        {                                                       \
            error_handler(ret_code, __FILE__, __LINE__);        \
        }                                                       \
    } while (0)

/** @brief A macro checking returning the error code if an error occurred. */
#define RETURN_ON_ERROR(ret_code)           \
    do {                                    \
        if (ret_code != RET_SUCCESS)        \
        {                                   \
            return ret_code;                \
        }                                   \
    } while (0)


/** @brief Error handler which exits the application.
 *
 *  @details If debug is enabled, some status information is printed.
 */
static inline void error_handler(ret_code_t ret_code,
                                 char * filename,
                                 int line_num)
{
#ifdef DEBUG
    LOG("Error handler:\nGot error: %d\nIn file %s\nLine %d\n", ret_code, filename, line_num);
#endif
    exit(ret_code);
}

#endif // APP_ERROR_H__
