#ifndef REQUEST_RESPONSE_H__
#define REQUEST_RESPONSE_H__

#include "ret_code.h"
#include "tcp_client.h"

/** @brief Issues a request.
 *
 *	@details This call is blocking. It returns when a response is received.
 *
 *  @param[in] 	p_request		The request to be sent.
 *  @param[out] p_response		The received response message.
 *
 *  @return RET_SUCCESS		If request was successful
 *  else return some error code.
 */
ret_code_t issue_request(message_t * p_request, message_t * p_response);


/** @brief A message handler.
 *
 * @details When a request is sent, the message server must forward responses to this module.
 * 			Therefore, this function must be called on a response message.
 *
 * @param[in] 	p_message		The received message.
 */
void request_response_on_response(message_t * p_message);

#endif // REQUEST_RESPONSE_H__
