#ifndef STATE_MACHINE_H__
#define STATE_MACHINE_H__

#include "ret_code.h"

/** @brief Definitions of the system states. */
typedef enum
{
    STATE_IDLE     = 0x01,  /**< The state indicating that the system is IDLE, that is the camera is deactivated and is not waiting for configuration data. */
    STATE_SCANNING = 0x02,  /**< The system is scanning frames from the camera and transmitting data to the host. */
    STATE_CONFIG   = 0x03   /**< The system is in configuration mode. In this mode the options used in the scanning state can be modified. */
} state_t;

/** @brief Commands the line scanner module to change state.
 *
 *	@param[in] state		The new state of the state machine.
 *
 *  @return RET_SUCCESS		If state successfully changed
 *  else return some error code.
 */
ret_code_t state_machine_goto_state(state_t state);

#endif
