#!/usr/bin/python

from JetsonTarget import JetsonTarget
from Plotters import *
from ProcessingTimeSaver import * 
import sys


def scan_mode(target):
	target.startScan()

	ans = True
	while ans:
		print("""
	1. Show height profile
	2. Show GPU processing time
	3. Show total processing time
	4. Show TCP sending time
	5. Show processing time combined
	6. Show colors
	7. Sample processing time
	Press Enter To Quit
		""")
		try:
			ans=input("What Would You Like To Do Now?: ")
		except:
			ans = ""
			pass
		if ans == 1:
			plotter = LinePlotter(get_max_height_func = target.getMaxHeight,
								get_num_scan_lines_per_image_func = target.getNumScanLinesPerImage,
								get_height_func=target.getHeight,
								get_size_func  =target.getScatter,
								get_color_func =target.getColor)
			
			plotter.show()
		elif ans == 2:
			plotter = ProcessTimePlotter(get_processing_time_func=target.getGPUProcessTime, title="GPU processing time")
			plotter.show()
		elif ans == 3:
			plotter = ProcessTimePlotter(get_processing_time_func=target.getTotalProcessTime, title="Total processing time")
			plotter.show()
		elif ans == 4:
			plotter = ProcessTimePlotter(get_processing_time_func=target.getTCPSendTime, title="TCP sending time")
			plotter.show()
		elif ans == 5:
			plotter = ProcessTimePlotterCombined(get_processing_time_func=target.getProcessTimeCombined, titles=["GPU processing time", "Total processing time"])
			plotter.show()
		elif ans == 6:
			plotter = ColorPlotter(get_color_func=target.getColor)
			plotter.show()
		elif ans == 7:
			sample_time = input("\tEnter how many seconds the sampling should take: ")
			file_name =  raw_input("Enter the file name: ")

			ProcessingTimeSaver(file_name, 
				get_callback_time_func=target.getCallbackTime,
				get_image_processing_time_func=target.getProcessTime, 
				sample_time_seconds=sample_time)
			
		else:
			break
	target.stopScan()

def get_image(target):
	target.getImage("my_img.png")

def main(args):
	if len(args) != 2:
		print "Usage: address port"
		return
	
	args[0] = str(args[0])
	args[1] = int(args[1])

	try:
		target = JetsonTarget(args[0], args[1])
	except Exception as e: 
		print(e)
		print "Could not connect to", args[0], ":", args[1]
		return

	print "Connected to target"

	ans = True
	while ans:
		print("""
	1. Start scanning
	2. Get camera image
	3. Plot saved processing time data
	Press Enter To Quit
		""")
		try:
			ans=input("What Would You Like To Do Now?: ")
		except:
			ans = ""
			pass

		if ans == 1:
			scan_mode(target)
		elif ans == 2:
			get_image(target)
		elif ans == 3:
			file_name =  raw_input("Enter the file name: ")
			SavedDataPlotter(file_name)
		else:
			break
		
if __name__ == "__main__":
   main(sys.argv[1:])
