import socket
from RequestResponse import *
import webbrowser
from threading import Thread

from collections import deque 

class JetsonTarget:

	requestHandler = None

	scanThread = None

	def enum(**enums):
		return type('Enum', (), enums)

	State = enum (	IDLE 		= 0x01,
					SCANNING 	= 0x02,
					CONFIG   	= 0x03)					

	MessageType = enum(	GOTO_STATE                 = 0x01,
						CONFIG_CAMERA              = 0x02,
						CONFIG_SCAN		           = 0x03,
						GET_IMAGE                  = 0x04,
						SCAN_DATA                  = 0x05,

						RESPONSE                   = 0x80)

	def __init__(self, ip, port):
		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		s.connect((ip, port))

		self.requestHandler = RequestResponse(s)
		self.stopThread = False

		self.scan_lines_per_image = None
		self.max_height		= None
		self.height 		= deque([], 4)
		self.intensity 		= deque([], 4)
		self.scatter 		= deque([], 4)
		self.reflectance	= deque([], 4)
		self.color			= deque([], 4)
		self.gpu_proc_time  = deque([], 4)
		self.total_proc_time= deque([], 4)
		
	def startScan(self):
		ret_code = self.__gotoState(self.State.SCANNING)
		if ret_code != 0:
			raise ValueError('Did not manage to enter scan state, got error code', ret_code)

		if self.scanThread == None:
			self.scanThread = Thread(target = self.__readScanDataThread)
			self.scanThread.start()

	def stopScan(self):
		if self.scanThread != None:
			self.stopThread = True
			self.scanThread.join()
			self.scanThread = None

		ret_code = self.__gotoState(self.State.IDLE)
		if ret_code != 0:
			raise ValueError('Did not manage to leave scan state')

	def getNumScanLinesPerImage(self):
		return self.scan_lines_per_image

	def getMaxHeight(self):
		return self.max_height

	def getHeight(self):
		if len(self.height) > 0:
			return self.height.popleft()
		else:
			return None
			
	def getIntensity(self):
		if len(self.intensity) > 0:
			return self.intensity.popleft()
		else:
			return None

	def getReflectance(self):
		if len(self.reflectance) > 0:
			return self.reflectance.popleft()
		else:
			return None

	def getScatter(self):
		if len(self.scatter) > 0:
			return self.scatter.popleft()
		else:
			return None

	def getGPUProcessTime(self):
		if len(self.gpu_proc_time) > 0:
			return self.gpu_proc_time.popleft()
		else:
			return None

	def getColor(self):
		if len(self.color) > 0:
			return self.color.popleft()
		else:
			return None

	def getTotalProcessTime(self):
		if len(self.total_proc_time) > 0:
			return self.total_proc_time.popleft()
		else:
			return None

	def getProcessTimeCombined(self):
		return [self.getGPUProcessTime(), self.getTotalProcessTime()]

	def getTCPSendTime(self):
		if len(self.gpu_proc_time) > 0 and len(self.total_proc_time) > 0:
			return self.getTotalProcessTime() - self.getGPUProcessTime()

	def getImage(self, file_name):
		self.__gotoState(self.State.CONFIG)
		(ret_code,data) = self.requestHandler.issueRequest(self.MessageType.GET_IMAGE, None)
		if ret_code == 0:
			myfile = open(file_name, "wb")
			myfile.write(data)
			myfile.close()
			webbrowser.open(file_name)
		self.__gotoState(self.State.IDLE)

	def __gotoState(self, State):
		(ret_code,data) = self.requestHandler.issueRequest(self.MessageType.GOTO_STATE, State)
		return ret_code

	def __readScanDataThread(self):
		oldTimestamp = None
		try:
			while self.stopThread == False:
				packed = list(self.requestHandler.getScanData())
				for returned in packed:
					self.scan_lines_per_image = returned[0]
					self.max_height     	  = returned[1]
					self.height.append(returned[2])
					self.intensity.append(returned[3])
					self.scatter.append(returned[4])
					self.reflectance.append(returned[5])
					self.color.append(returned[6])
					if returned[7] != None:
						self.gpu_proc_time.append(returned[7])
					if returned[8] != None:
						self.total_proc_time.append(returned[8])
		except Exception as e:
			print e
			self.__gotoState(self.State.IDLE)

