#ifndef IMG_PROCESSING_H__
#define IMG_PROCESSING_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "ret_code.h"
#include "stream_data.h"

typedef scan_config_s scan_config_t;


/** @brief Sets the configuration of the image processing module.
 *
 *  @details The configuration is first validated before it is set. This function call modifies the internal state.
 *
 *  @param[in] p_config                 The image processing configuration.
 *
 *  @return RET_SUCCESS                 If the pin was successfully configured for output.
 *  @return RET_ERROR_NULL              If the provided parameter was NULL.
 *  @return RET_ERROR_INVALID_CONFIG    If specified configuration is invalid.
 */
ret_code_t img_processing_set_config(scan_config_t * p_config);


/** @brief Starts an image processing task.
 *
 *  @details This function asynchronously starts memory transfers, the corresponding CUDA kernel,
 *           and a TCP send callback. In addition the previously used time is stored and sent over TCP.
 *
 *  @param[in] p_stream_data            The data corresponding to the image processing task.
 *
 *  @return RET_SUCCESS                 If the pin was successfully configured for output.
 *  @return RET_ERROR_NULL              If the provided parameter was NULL.
 *  @return RET_ERROR_NOT_INITIALIZED   If image processing module is not initialized.
 *  @return RET_ERROR_OTHER             If an internal error occurred.
 */
ret_code_t img_processing_start(stream_data_t  * p_stream_data);


#ifdef __cplusplus
}
#endif

#endif
