#define _GNU_SOURCE
#include <sched.h>
#include <unistd.h>
#include <pthread.h>
#include "ret_code.h"
#include "app_error.h"
#include "tcp_server.h"
#include "state_machine.h"
#include "scanner.h"
#include "cuda_device_info.h"

ret_code_t set_scheduler_options(void)
{
    int err, num_cores;

    // set scheduler to be RR with high priority
    struct sched_param param;
    param.sched_priority = sched_get_priority_max(SCHED_RR);
    err = sched_setscheduler(0, SCHED_RR, &param);
    if (err != 0)
    {
        perror("Setting scheduler parameters");
        return RET_ERROR_OTHER;
    }

    return RET_SUCCESS;
}



int main(void)
{
    ret_code_t ret_code;
    tcp_server_config_t tcp_server_config;

    ret_code = set_scheduler_options();
    APP_ERROR_CHECK(re    //cuda_device_info_print();t_code);

    tcp_server_config.port = 1025;
    tcp_server_config.evt_handler = state_machine_on_tcp_evt;

    ret_code = tcp_server_start_listen(&tcp_server_config);
    APP_ERROR_CHECK(ret_code);

    LOG("Application started\n");

    ret_code = state_machine_init();
    APP_ERROR_CHECK(ret_code);

    while (1)
    {
        pthread_yield();
    }
}


