#include <math.h>
#include "frame_buffer.h"
#include "scanner.h"
#include "log.h"

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define GET_INDEX(column, height, offset) ((offset) + (column) +  PITCH*(height))


ret_code_t scan_mode_2_init(scan_config_t * p_config)
{
    return RET_SUCCESS;
}



__global__
void scan_mode_2_kernel(frame_buffer_t * d_frame_buffers,
                        scan_output_buffer_t * d_output_buffer)
{
    const uint16_t * p_color_data = (uint16_t *)d_frame_buffers[0].d_image_buffer;

    const uint32_t NUM_SCAN_LINES_PER_IMAGE = d_frame_buffers[0].s_width / (sizeof(uint16_t) * 2);
    const uint32_t MAX_HEIGHT               = d_frame_buffers[0].s_height;
    const uint16_t PITCH                    = d_frame_buffers[0].s_width / sizeof(uint16_t);

    const uint32_t image_index = blockIdx.y;
    const uint32_t local_scan_line = blockIdx.x*blockDim.x + threadIdx.x;
    const uint32_t scan_line = image_index * NUM_SCAN_LINES_PER_IMAGE + local_scan_line;

    const uint32_t column = local_scan_line * 2;
    const uint32_t image_offset = image_index * PITCH * MAX_HEIGHT;

    if (local_scan_line >= NUM_SCAN_LINES_PER_IMAGE)
    {
        return;
    }

    d_output_buffer->p_height_buffer[scan_line]      = 0;
    d_output_buffer->p_reflectance_buffer[scan_line] = 0;
    d_output_buffer->p_intensity_buffer[scan_line]   = 0;
    d_output_buffer->p_scatter_buffer[scan_line]     = 0;

    const uint32_t r_index = scan_line * 3;
    const uint32_t g_index = r_index + 1;
    const uint32_t b_index = r_index + 2;

    d_output_buffer->p_rgb_buffer[r_index]
          = p_color_data[image_offset + column];

    d_output_buffer->p_rgb_buffer[g_index]
          = p_color_data[image_offset + column + 1];

    d_output_buffer->p_rgb_buffer[b_index]
          = p_color_data[image_offset + column + PITCH + 1];
}

