#include <TimerOne.h>

#define SCAN_MODE             4

#define USE_JETSON_READY_PIN  false
#define DUTY_CYCLE_MICROSEC   5000

#define IMAGE_CONTENT_PIN_0 2 
#define IMAGE_CONTENT_PIN_1 3

#define CAMERA_TRIGGER_PIN  7

#define LASER_PIN           4
#define LED_PIN             6




#define JETSON_READY_PINS   {8}

typedef enum
{
    IMAGE_CONTENT_LINE   = 0,
    IMAGE_CONTENT_BLANK  = 1,
    IMAGE_CONTENT_COLOR  = 2
} image_content_t;

// The scan modes represent the length and type of the image
#define IMG_SEQ_LENGTH_OFFSET   0x80
#define IMG_SEQ_LENGTH_INDEX    0
#define IMG_SEQ_START_INDEX     1
#define IMG_SEQ_PADDING         0xFF

#define IMG_SEQ_0 {1 + IMG_SEQ_LENGTH_OFFSET, IMAGE_CONTENT_LINE,  IMG_SEQ_PADDING,     IMG_SEQ_PADDING}
#define IMG_SEQ_1 {2 + IMG_SEQ_LENGTH_OFFSET, IMAGE_CONTENT_LINE,  IMAGE_CONTENT_COLOR, IMG_SEQ_PADDING}
#define IMG_SEQ_2 {1 + IMG_SEQ_LENGTH_OFFSET, IMAGE_CONTENT_COLOR, IMG_SEQ_PADDING,     IMG_SEQ_PADDING}
#define IMG_SEQ_3 {2 + IMG_SEQ_LENGTH_OFFSET, IMAGE_CONTENT_LINE,  IMAGE_CONTENT_BLANK, IMG_SEQ_PADDING}
#define IMG_SEQ_4 {3 + IMG_SEQ_LENGTH_OFFSET, IMAGE_CONTENT_LINE,  IMAGE_CONTENT_BLANK, IMAGE_CONTENT_COLOR}

static const uint8_t IMG_SEQUENCES[][4] = {IMG_SEQ_0,
                                           IMG_SEQ_1,
                                           IMG_SEQ_2,
                                           IMG_SEQ_3,
                                           IMG_SEQ_4,
                                           IMG_SEQ_0,};

#define IMG_SEQ_GET_LENGTH(scan_mode)         (IMG_SEQUENCES[(scan_mode)][IMG_SEQ_LENGTH_INDEX] - IMG_SEQ_LENGTH_OFFSET)
#define IMG_SEQ_GET_IMG_TYPE(scan_mode, idx)  ((image_content_t)IMG_SEQUENCES[(scan_mode)][(idx)+1])


static uint8_t jetson_ready_pins[] = JETSON_READY_PINS;

// the setup function runs once when you press reset or power the board
void setup() {
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(CAMERA_TRIGGER_PIN, OUTPUT);
  pinMode(IMAGE_CONTENT_PIN_0, OUTPUT);
  pinMode(IMAGE_CONTENT_PIN_1, OUTPUT);
  pinMode(LASER_PIN, OUTPUT);
  pinMode(LED_PIN, OUTPUT);

#if USE_JETSON_READY_PIN
  for (uint8_t i = 0; i < sizeof(jetson_ready_pins); i++)
  {
    pinMode(jetson_ready_pins[i], INPUT);
  }
#endif

  Timer1.initialize(DUTY_CYCLE_MICROSEC);
  Timer1.attachInterrupt(timerCallback);
}


// the loop function runs over and over again forever
void loop() 
{
  //camera_trigger();
  //delayMicroseconds(1500);
}




static void camera_set_next_frame_type(void)
{
  static uint8_t img_type_index = 0;
  
  switch(IMG_SEQ_GET_IMG_TYPE(SCAN_MODE, img_type_index))
  {
    case IMAGE_CONTENT_LINE:
      digitalWrite(LASER_PIN,           HIGH);
      digitalWrite(LED_PIN,             LOW);
      digitalWrite(IMAGE_CONTENT_PIN_0, LOW);
      digitalWrite(IMAGE_CONTENT_PIN_1, LOW);
      break;
      
    case IMAGE_CONTENT_BLANK:
      digitalWrite(LASER_PIN,           LOW);
      digitalWrite(LED_PIN,             LOW);
      digitalWrite(IMAGE_CONTENT_PIN_0, HIGH);
      digitalWrite(IMAGE_CONTENT_PIN_1, LOW);
      break;

    case IMAGE_CONTENT_COLOR:
      digitalWrite(LASER_PIN,           LOW);
      digitalWrite(LED_PIN,             HIGH);
      digitalWrite(IMAGE_CONTENT_PIN_0, LOW);
      digitalWrite(IMAGE_CONTENT_PIN_1, HIGH);
      break;
  }
    
  const uint8_t image_sequence_length = IMG_SEQ_GET_LENGTH(SCAN_MODE);
  // switch to the next image type in the image sequence
  if (img_type_index < image_sequence_length - 1)
  {
      img_type_index++;
  }
  else
  {
      img_type_index = 0;
  } 
}

static void camera_trigger(void)
{
#if USE_JETSON_READY_PIN
  for (uint8_t i = 0; i < sizeof(jetson_ready_pins); i++)
  {
    while (digitalRead(jetson_ready_pins[i]) == 0) ;
  }
#endif
 
  digitalWrite(CAMERA_TRIGGER_PIN, LOW);
  delayMicroseconds(500);
  digitalWrite(CAMERA_TRIGGER_PIN, HIGH);
}


static void timerCallback(void)
{
  camera_set_next_frame_type();
  camera_trigger();
}

