#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif
#include "ret_code.h"
#include "app_error.h"
#include "log.h"
#include "tcp_client.h"

#include "camera_config.h"
#include "scan_config.h"
#include "request_response.h"
#include "scan_data_parser.h"
#include "state_machine.h"

#define TARGET_PORT    1025
#define TARGET_IP      "10.0.0.2"

void on_tcp_evt(tcp_evt_t * p_evt)
{
	switch (p_evt->type)
	{
		case TCP_EVT_TYPE_CONNECTED:
			LOG("Connected\n");
			break;

		case TCP_EVT_TYPE_DISCONNECTED:
			LOG("Disconnected\n");
			exit(1);
			break;

		case TCP_EVT_TYPE_RECV_MESSAGE:
			switch (p_evt->message.type)
			{
				case MSG_TYPE_RESPONSE:
					request_response_on_response(&p_evt->message);
					break;

				case MSG_TYPE_SCAN_DATA:
					break;

				default:
					LOG_DEBUG("Unexpected message type received: %d\n", p_evt->message.type);
					break;
			}
			break;
	}
}




int main(int argc, char *argv[])
{
    ret_code_t ret_code;

    tcp_client_config_t client_config;

    client_config.evt_handler   = on_tcp_evt;
    client_config.port          = TARGET_PORT;
    client_config.p_ip_address  = TARGET_IP;

    ret_code = tcp_client_connect(&client_config);
    APP_ERROR_CHECK(ret_code);

    ret_code = state_machine_goto_state(STATE_CONFIG);
    APP_ERROR_CHECK(ret_code);

    ret_code = camera_config_send_all();
    APP_ERROR_CHECK(ret_code);
    ret_code = scan_config_send_all();
    APP_ERROR_CHECK(ret_code);


    //print_config_data();

    ret_code = state_machine_goto_state(STATE_IDLE);
    APP_ERROR_CHECK(ret_code);

    ret_code = tcp_client_disconnect();
    APP_ERROR_CHECK(ret_code);

    LOG_DEBUG("Disconnected\n");

    return 0;
}
