#include "request_response.h"

#include <string.h>
#include <stdbool.h>
#include "app_error.h"
#include "log.h"

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif


volatile bool 			request_barrier;
volatile message_type_t send_request_type;
message_t 		*p_received_msg;


ret_code_t issue_request(message_t * p_request, message_t * p_response)
{
	if (p_request == NULL || p_response == NULL)
	{
		return RET_ERROR_NULL;
	}

	ret_code_t ret_code;

	// set pointer to destination message
	send_request_type = p_request->type;
	p_received_msg = p_response;

	ret_code = tcp_client_send_message(p_request);
	RETURN_ON_ERROR(ret_code);

	while (request_barrier == false)
	{
#ifdef WIN32
		Sleep(1);
#else
		usleep(100);
#endif
	}

	request_barrier = false;

	return RET_SUCCESS;
}


void request_response_on_response(message_t * p_message)
{
	if (p_message->type != MSG_TYPE_RESPONSE)
	{
		APP_ERROR_HANDLER(RET_ERROR_INVALID_PARAMETER);
	}

	if (p_message->response.request_type != send_request_type)
	{
		APP_ERROR_HANDLER(RET_ERROR_INVALID_PARAMETER);
	}

	memcpy(p_received_msg, p_message, sizeof(message_t));

	request_barrier = true;
}

